#!/usr/bin/perl

use strict;
my %flags = ('force'=>0, 'verbose'=>0, 'help'=>0, 'quiet'=>0, 'act'=>0, 'debug'=>0);
map {/^-(.)$/ ? ((grep /^$1/, keys %flags)[0] =~ /(.+)/) : /^--(\w+)$/; $flags{$1}++ if exists $flags{$1}} @ARGV;
my $VZCP_CONF_D = '/vz/root/1/etc/vzcp';
die 'Name:
	ssl_cert_vzplesk.pl - setup SSL certificates for Plesk under Virtuozzo

Synopsis:
	ssl_cert_vzplesk.pl [-h|-a [-v] [-d] [-q] [-f]]

Description:
	This program rebuilds the configuration of Virtuozzo CP to include SSL
	certificates from all Plesk containers running on Virtuozzo node with
	enabled Offline Management. Applies both to Virtuozzo 3.0 and 4.0. It
	should be run every time you add a new container with Plesk or reinstall
	a certificate in a Plesk container, to rebuild the configuration of
	Parallels Power Panel/Control Center and include the changed/added
	certificate.

Options:
	-h	--help		this help screen
	-a	--act		rebuild the configuration [mandatory]
	-v	--verbose	display diagnostic
	-d	--debug		include configuration files parsing
	-q	--quiet		be quiet, suppress output
	-f	--force		proceed with reconfiguration silently

Addendum:
	To apply the changes, add to the file \''.$VZCP_CONF_D.'/httpd.conf\'
	the following line to the end of file or right after the line with
	including the file "/etc/vzcp/addon_httpd_conf/plesk.conf":
		Include /etc/vzcp/addon_httpd_conf/plesk-ssl.conf

	Restart the service of Virtuozzo CP:
		# vzctl exec2 1 service vzcp restart

' if $flags{'help'};
die ($flags{'quiet'} == 2 ? '' : 'Usage: ssl_cert_vzplesk.pl [-h|-a [-v] [-d] [-q] [-f]]
') if !$flags{'act'};
if(!$flags{'force'})
{
    printf STDERR 'NOTE:
This program is going to rebuild the configuration for Parallels Power
Panel/Control Center to include SSL certificates from all Plesk Container
with enabled Offline Management. Do you want to proceed? (Hit Ctrl-C to
exit, Enter to proceed):
> ';
    <STDIN>;
}

sub getFileContentArr { open FH, "<".$_[0] or die $!.' ==> '.$_[0]; my @fc = <FH>; close FH; return @fc }
sub getConfValues { my ($fn, $delim, %conf) = @_; $fn = readlink $fn if -l $fn; open FH, "<".$fn or die $!.' ==> '.$fn; while(<FH>) { print STDERR "\tParse confline: $_" if $flags{'debug'}; $conf{$1}=$3 if m/^[[:space:]]*(\w+)[[:space:]]*$delim[[:space:]]*([\'\"]|)(.*?)\2[[:space:]]*?$/ } close FH; return %conf }
sub copyFile { open FH, "<".$_[0] or return -1; open FO, ">".$_[1] or return -2; my $buf; print FO $buf while(read FH, $buf, 65536); close FH; close FO; return 0 }

my $VZ_CONF_D = '/etc/sysconfig';
my $VZCPP_CONF_T = join '', grep !/^Listen/, getFileContentArr $VZCP_CONF_D.'/addon_httpd_conf/plesk.conf.template';
my $VZCPP_CONF_F = '';
my %VZ_CONF_F = getConfValues $VZ_CONF_D.'/vz', '=';
my @CTIDs = grep /./, split /[[:space:]]+/, `vzlist -a1`;
my $ct;
foreach $ct (@CTIDs)
{
    my %CT_CONF_F = getConfValues $VZ_CONF_D.'/vz-scripts/'.$ct.'.conf', '=', %VZ_CONF_F;
    my @IP_ADDRESS = grep !/^(0\.0\.0\.0)?$/, (split /[[:space:]]+/, $CT_CONF_F{'IP_ADDRESS'});
    print STDERR "Checking CT #$ct.\n" if $flags{'verbose'};
    next if ! (
	($CT_CONF_F{'OFFLINE_MANAGEMENT'} == 'yes') &&
	(grep /^vzpp-plesk$/, (split /[[:space:]]+/, $CT_CONF_F{'OFFLINE_SERVICE'})) &&
	$IP_ADDRESS[0]);
    my $CT_PRIVATE_D = $CT_CONF_F{'VE_PRIVATE'};
    $CT_PRIVATE_D =~ s/\$VEID/$ct/;
    $CT_PRIVATE_D .= ((-d $CT_PRIVATE_D.'/fs') ? '/fs' : '').'/root';
    print STDERR "\tLooking for Plesk.\n" if $flags{'verbose'};
    next if ! -f $CT_PRIVATE_D.'/etc/psa/psa.conf';
    my %PSA_CONF_F = getConfValues $CT_PRIVATE_D.'/etc/psa/psa.conf', '[[:space:]]';
    my $PRODUCT_CONF_D = $CT_PRIVATE_D.$PSA_CONF_F{'PRODUCT_ROOT_D'}.'/admin/conf';
    print STDERR "\tGetting a server name and the certificate of Plesk in $PSA_CONF_F{'PRODUCT_ROOT_D'}.\n" if $flags{'verbose'};
    next if ! (
	(-f $PRODUCT_CONF_D.'/httpsd.conf') &&
	(-f $PRODUCT_CONF_D.'/httpsd.pem'));
    my $PSA_SERVERNAME = (grep /./, (map {(split /[[:space:]]+/)[1]} grep /^ServerName/, getFileContentArr $PRODUCT_CONF_D.'/httpsd.conf'), 'plesk-ct'.$ct)[0];
    my %res;
    $res{'str'} = (sprintf "#CT%d/%s [%s] %s", $ct, $PSA_SERVERNAME, (join ', ', @IP_ADDRESS), (-f $PRODUCT_CONF_D.'/rootchain.pem')).$VZCPP_CONF_T;
    $res{'ssl'} = sprintf "  SSLCertificateFile \"/etc/vzcp/plesk-%d.pem\"\n  SSLCertificateKeyFile \"/etc/vzcp/plesk-%d.pem\"\n", $ct, $ct;
    $res{'ips'} = join ' ', map {$_.':8443'} @IP_ADDRESS;
    print STDERR "\tHostname ${PSA_SERVERNAME}, conffile ${PRODUCT_CONF_D}/httpsd.conf, certfile ${PRODUCT_CONF_D}/httpsd.pem.\n" if $flags{'verbose'};
    printf STDERR "warning: cannot copy certificate '%s'\n", $PRODUCT_CONF_D.'/httpsd.pem' if copyFile($PRODUCT_CONF_D.'/httpsd.pem', $VZCP_CONF_D.'/plesk-'.$ct.'.pem') && !$flags{'quiet'};
    if (-f $PRODUCT_CONF_D.'/rootchain.pem')
    {
	printf STDERR "warning: cannot copy root chain certificate '%s'\n", $PRODUCT_CONF_D.'/rootchain.pem' if copyFile($PRODUCT_CONF_D.'/rootchain.pem', $VZCP_CONF_D.'/plesk-'.$ct.'-rootchain.pem') && !$flags{'quiet'};
	$res{'ssl'} .= sprintf "  SSLCACertificateFile \"/etc/vzcp/plesk-%d-rootchain.pem\"\n", $ct;
	printf STDERR "\tAdding a root chain certificate.\n" if $flags{'verbose'};
    }
    printf STDERR "Configuring CT #%d \'%s\' with IPs [%s]\n\n", $ct, $PSA_SERVERNAME, (join ', ', @IP_ADDRESS) if $flags{'verbose'};
    $res{'str'} =~ s~<VirtualHost .+>~<VirtualHost $res{'ips'}>\n  ServerName $PSA_SERVERNAME~;
    $res{'str'} =~ s~(SSLEngine .*)~\1\n$res{'ssl'}~;
    $res{'str'} =~ s~(set X_VZCP_PSA_PORT) .*~\1 8443~;
    $res{'str'} =~ s~(SetEnv VZCP_PORT) .*~\1 8443~;
    $VZCPP_CONF_F .= $res{'str'}."\n";
}
if(! open FH, '>'.($VZCP_CONF_D.'/addon_httpd_conf/plesk-ssl.conf.new'))
{
    print $VZCPP_CONF_F;
    exit -1;
}
print FH $VZCPP_CONF_F;
close FH;
rename $VZCP_CONF_D.'/addon_httpd_conf/plesk-ssl.conf', $VZCP_CONF_D.'/addon_httpd_conf/plesk-ssl.conf~backup' or $flags{'quiet'} or printf STDERR "warning: cannot rename old file '%s'\n", $VZCP_CONF_D.'/addon_httpd_conf/plesk-ssl.conf';
rename $VZCP_CONF_D.'/addon_httpd_conf/plesk-ssl.conf.new', $VZCP_CONF_D.'/addon_httpd_conf/plesk-ssl.conf' or $flags{'quiet'} or printf STDERR "warning: cannot rename new file to '%s'\n", $VZCP_CONF_D.'/addon_httpd_conf/plesk-ssl.conf';
printf STDERR "\nDo not forget to add into '%s' the following line after including 'plesk.conf':\n\tInclude /etc/vzcp/addon_httpd_conf/plesk-ssl.conf\n\nand restart the service:\n\t# vzctl exec2 1 service vzcp restart\n\n", $VZCP_CONF_D.'/httpd.conf' if !$flags{'quiet'};
